/*
 * Programmet tar utgangspunkt i et tidligere utviklet program.
 */
package client;

import javax.swing.*;
import java.io.*;
import java.net.Socket;
import java.net.SocketException;
import java.util.Arrays;

public class Client extends JFrame {
    private final JTextArea displayArea;
    private Socket connection;
    private DataOutputStream output;
    private DataInputStream input;

    Client() {
        super("DebugClient");

        setLayout(null);
        setSize(720, 480);
        setResizable(false);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        JTextArea input = new JTextArea();

        JScrollPane inputPane = new JScrollPane(input);
        inputPane.setBounds(10, 10, 570, 100);

        JButton connect = new JButton("Connect");
        connect.setBounds(590, 10, 100, 40);
        connect.addActionListener(e -> connect());

        JButton disconnect = new JButton("Disconnect");
        disconnect.setBounds(590, 70, 100, 40);
        disconnect.addActionListener(e -> disconnect());

        JButton send = new JButton("Send");
        send.setBounds(590, 130, 100, 40);
        send.addActionListener(e -> sendMessage(hexStringToByteArray(input.getText())));

        displayArea = new JTextArea();
        displayArea.setEditable(false);
        JScrollPane pane = new JScrollPane(displayArea);
        pane.setBounds(10, 120, 570, 280);

        getContentPane().add(inputPane);
        getContentPane().add(connect);
        getContentPane().add(disconnect);
        getContentPane().add(send);
        getContentPane().add(pane);
        setVisible(true);
    }


    public void disconnect() {
        try {
            output.close();
            input.close();
            connection.close();
        } catch (SocketException | NullPointerException e) {
            System.out.println("Connection closed");
        } catch (IOException e) {
            e.printStackTrace();
        }

        connection = null;
    }

    public static byte[] hexStringToByteArray(String s) {
        int len = s.length();
        byte[] data = new byte[len / 2];
        for (int i = 0; i < len; i += 2) {
            data[i / 2] = (byte) ((Character.digit(s.charAt(i), 16) << 4)
                    + Character.digit(s.charAt(i + 1), 16));
        }
        return data;
    }

    public void connect() {
        byte[] hand = new byte[]{(byte) 0x00, (byte) 0x02, (byte) 0x30, (byte) 0x31};
        if (connection != null) return;
        try {
            String IP = "localhost";
            println("Connecting to " + IP + " on port:" + 8160);
            connection = new Socket(IP, 8160);
            output = new DataOutputStream(connection.getOutputStream());
            input = new DataInputStream(connection.getInputStream());
            println("Connected\n");
        } catch (IOException e) {
            println("Failed");
            e.printStackTrace();
        }
        sendMessage(hand);
    }

    private void listenForResponse() {
        try {
            byte[] bytes = new byte[4096];
            var i = input.read(bytes, 0, 4096);

            if (i <= 0) {
                System.out.println("Klarte ikke å lese data");
                disconnect();
                return;
            }

            byte[] data = new byte[i];
            System.arraycopy(bytes, 0, data, 0, i);

            println("MESSAGE FROM SERVER:");
            println(Arrays.toString(data) + "\n");
        } catch (IOException e) {
            println("Lost connection to server");
            disconnect();
        }
    }

    private void sendMessage(byte[] data) {
        try {
            output.write(data);
            output.flush();
            println("MESSAGE SENT:");
            println(data + "\n");
            listenForResponse();
        } catch (IOException e) {
            println("Lost connection to server");
            disconnect();
        }
    }


    private void println(String str) {
        displayArea.append(str + "\n");
    }


    public static void main(String[] args) {
        Client client = new Client();
    }
}
