package simulation;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.util.Arrays;

public class SimulationClient extends Thread {
    private Socket connection;
    private DataOutputStream output;
    private DataInputStream input;

    private final String address;
    private final int port;
    private boolean connected;

    SimulationClient(String address, int port) {
        this.address = address;
        this.port = port;
        setDaemon(true);
    }

    public boolean isConnected() {
        return connected;
    }

    public static byte[] hexStringToByteArray(String s) {
        int len = s.length();
        byte[] data = new byte[len / 2];
        for (int i = 0; i < len; i += 2) {
            data[i / 2] = (byte) ((Character.digit(s.charAt(i), 16) << 4)
                    + Character.digit(s.charAt(i + 1), 16));
        }
        return data;
    }

    public boolean connect(String handshake) {
        try {
            connection = new Socket(address, port);
            output = new DataOutputStream(connection.getOutputStream());
            input = new DataInputStream(connection.getInputStream());
            sendMessage(handshake);
            byte[] response = listenForResponse();
            if (response.length == 1 && response[0] > 0) {
                connected = true;
                return true;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return connected;
    }

    public void disconnect() {
        try {
            output.close();
            input.close();
            connection.close();
        } catch (NullPointerException | IOException e) {
            e.printStackTrace();
        }
        connected = false;
    }

    private byte[] listenForResponse() {
        try {
            byte[] bytes = new byte[4096];
            var i = input.read(bytes, 0, 4096);

            if (i <= 0) throw new IOException("Klarte ikke å lese data");

            byte[] data = new byte[i];
            System.arraycopy(bytes, 0, data, 0, i);
            return data;

        } catch (IOException e) {
            disconnect();
            e.printStackTrace();
        }

        return new byte[]{};
    }

    public void sendMessage(String str) {
        byte[] data = hexStringToByteArray(str);
        try {
            output.write(data);
            output.flush();
        } catch (IOException e) {
            disconnect();
            e.printStackTrace();
        }
    }

    @Override
    public void run() {
        while (connected) System.out.println(Arrays.toString(listenForResponse()));
        disconnect();
    }
}
