package simulation.avl;

import simulation.Crc;

import java.util.ArrayList;

public class Avl {
    private final String preamble;
    private final String codec;
    private final ArrayList<AvlPackage> avlPackages;
    private String crc;

    public Avl() {
        this.preamble = "00000000";
        this.codec = "8E";
        this.avlPackages = new ArrayList<>();
        this.crc = "00000000";
    }

    public static String intToHex(int val, int size) {
        var a = "0".repeat(size * 2);
        var hex = Integer.toHexString(val);
        String out = (a + hex).substring(hex.length());
        return out.toUpperCase();
    }

    public static String longToHex(long val, int size) {
        var a = "0".repeat(size * 2);
        var hex = Long.toHexString(val);
        String out = (a + hex).substring(hex.length());
        return out.toUpperCase();
    }

    public static String stringToHex(String str) {
        StringBuilder sb = new StringBuilder();
        char[] ch = str.toCharArray();
        for (char c : ch) {
            String hexString = Integer.toHexString(c);
            sb.append(hexString);
        }
        return sb.toString();
    }

    int getAvlLength() {
        int codecSize = 1;
        int avlCountSize = 1;
        return (int) (Math.ceil(this.getAvlHex().length() / 2f) + codecSize + avlCountSize * 2);
    }

    String getAvlLengthHex() {
        return intToHex(this.getAvlLength(), 4);
    }

    String getAvlHex() {
        String out = "";
        for (AvlPackage avlPackage : avlPackages) out += avlPackage;
        return out;
    }

    String getAvlCountHex() {
        return intToHex(this.avlPackages.size(), 1);
    }

    void updateCrc() {
        String str = this.toString();
        byte[] data = new byte[(str.length()) / 2];
        int idx = 0;
        for (var i = 0; i < str.length(); i += 2)
            data[idx++] = (byte) Integer.parseInt(str.substring(i, i + 2), 16);
        this.crc = intToHex(Crc.crcChecker(data), 4);
    }

    public void addAvlPackage(AvlPackage avlPackage) {
        this.avlPackages.add(avlPackage);
        updateCrc();
    }

    @Override
    public String toString() {
        return this.preamble +
                this.getAvlLengthHex() +
                this.codec +
                this.getAvlCountHex() +
                this.getAvlHex() +
                this.getAvlCountHex() +
                this.crc;
    }
}
