package dataprocessing;

import dataprocessing.output.Output;
import networking.Packet;

/**
 * Base klasse for alle protokoller.
 * Har som formål å tolke rå data fra sensor til felles Output-klasse.
 * Sender data til database-behandler
 */
public interface Protocol {
    enum Codes{
        ERROR,
        DATA,
        HANDSHAKE
    }

    /**
     * Funksjon for validering og klassifisering av Data.
     * Implementeres i henhold til enhetenfamiliens protokoll
     *
     * @param packet pakken som skal valideres og klassifiseres
     * @return Klassifikasjonskoden til pakken
     */
    Codes validateData(Packet packet);

    /**
     * Funksjon for å tolke handshake pakker. Skal returnere IDen til enheten
     *
     * @param packet Handskakepakken
     * @return IDen eller -1 hvis noe var galt
     */
    long processHandshake(Packet packet);

    /**
     *  Funksjon for tolkning av datapakker. Tolker data etter enhetsfamilien sin protokoll for datatolkning
     *
     * @param packet Datapakken
     * @return tolket dataen i felles Output klasse. Er det noe galt under tolkning skal null returnes
     * @see Output
     */
    Output processData(Packet packet);
}
