package dataprocessing.output;

import dataprocessing.output.equipment.Equipment;

import java.util.ArrayList;
import java.util.List;

/**
 * Klasse som inneholder alle PacketData lest ved en innsending
 */
public class Output {
    private String deviceFamily;
    private final long deviceId;
    private final ArrayList<PacketContainer> packetContainers;

    /**
     * @param deviceId ID-en til enheten som sendte dataen
     */
    public Output(long deviceId) {
        this.deviceId = deviceId;
        deviceFamily = "Ukjent";
        packetContainers = new ArrayList<>();
    }

    /**
     * @return Enhets_ID
     */
    public long getDeviceId() {
        return deviceId;
    }

    /**
     * Setter navn på enhetsfamilen
     *
     * @param family navn på familien
     */
    public void setDeviceFamily(String family) {
        deviceFamily = family;
    }

    /**
     * @return enhetsfamilien
     */
    public String getDeviceFamily() {
        return deviceFamily;
    }

    /**
     * Legg til en ny PacketContainer
     *
     * @param container PacketConatiner
     */
    public void addDataPacket(PacketContainer container) {
        packetContainers.add(container);
    }

    /**
     * @return listen med PacketContainere
     */
    public List<PacketContainer> getDataPackets() {
        return packetContainers;
    }

    /**
     * @return Antall PacketContainere tolket
     */
    public int getLength() {
        return packetContainers.size();
    }

    @Override
    public String toString() {
        StringBuilder out;
        out = new StringBuilder("\nDevice Family: " + deviceFamily
                + "\n\nDatapackets (" + packetContainers.size() + "):");
        for (PacketContainer d : packetContainers) {
            out.append("\n\nDatapacket no. ").append(packetContainers.indexOf(d) + 1).append("\n\t\tDate: ").append(d.getDateAndTime()).append("\n\n\tGPS-Data: ").append(d.getGpsElement()).append("\n\n\tEquipment:");

            for (Equipment e : d.getEquipmentArray()) {
                out.append(e);
            }

            out.append("\n\n");
        }
        return out.toString();
    }
}
