package dataprocessing.output;

import dataprocessing.output.equipment.Equipment;

import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;

/**
 * Klasse som inneholder tolket data sendt fra en enhet
 */
public class PacketContainer {
    private long timeStamp;
    private GpsElement gpsElement;
    private final HashMap<String, Equipment> equipment;

    public PacketContainer() {
        equipment = new HashMap<>();
    }

    /**
     * Lager et GPS-objekt fra innhentet data
     *
     * @param longitude lengdegraden til posisjonen
     * @param latitude breddegraden til posisjonen
     * @param altitude høyden til posisjonen
     * @param angle vinkelen til posisjonen
     * @param noOfSatellites Antall satelitter ved måling
     */
    public void setGpsElement(double longitude, double latitude, short altitude, short angle, byte noOfSatellites) {
        gpsElement = new GpsElement(longitude, latitude, altitude, angle, noOfSatellites);
    }

    /**
     * @return GPS-objektet
     */
    public GpsElement getGpsElement() {
        return gpsElement;
    }

    /**
     * @return Henter liste med Equipment-data (IOevent-data)
     */
    public Equipment[] getEquipmentArray() {
        return equipment.values().toArray(new Equipment[0]);
    }

    /**
     * Setter tiden pakken ble generert
     * @param datetime tidsstempelet
     */
    public void setDateAndTime(long datetime) {
        timeStamp = datetime;
    }

    /**
     * @return Tidsstempelet
     */
    public long getDateAndTime() {
        return timeStamp;
    }

    /**
     * Henter utstyr etter gitt klasse. Finnes det ingen av dette utstyret i listen lagges det ett nytt et, som returneres
     *
     * @param c   Klassen til det ønskede utstyret
     * @param <T> Klassentypen til det ønskede utstyret
     * @return Utstyr objektet som tilhører datapakken
     * @throws NoSuchMethodException
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     * @throws InstantiationException
     */
    public <T extends Equipment> T getEquipment(Class<T> c) throws  ReflectiveOperationException {
        if (!equipment.containsKey(c.getName()))
            equipment.put(c.getName(), c.getConstructor().newInstance());
        return c.cast(equipment.get(c.getName()));
    }
}
