package networking.teltonikafmc;

import dataprocessing.output.Output;
import networking.Packet;
import networking.Client;
import networking.Server;
import tools.DataConverter;

import java.io.*;
import java.net.Socket;

/**
 * TeltonikaFMC klient
 */
public class TeltonikaFmcClient extends Client {
    public TeltonikaFmcClient(Server handler, Socket socket, DataInputStream input, DataOutputStream output) {
        super(handler, socket, input, output, 1280);
    }

    /**
     * TeltonikaFMC enheter forventer å få enten 1 eller 0 som respons ved handshake.
     * Sender 1 ved godkjent og 0 ved ikke godkjent
     *
     * @param flag Boolean som sier om klienten ble godkjent eller ikke
     */
    @Override
    public void onAcceptConnection(boolean flag) {
        send(DataConverter.int8ToByte((byte) (flag ? 1 : 0)));
    }

    /**
     * TeltonikaFMC enheter forventer antall mottatte AVL-datapakker i respons ved vanlige pakker
     * Sender antall datapakker som har blit mottatt
     *
     * @param data Dataen som har blitt tolket
     */
    @Override
    public void onSendReceipt(Output data) {
        send(DataConverter.int32ToByte(data != null ? data.getLength() : 0));
    }

    /**
     * Omgjør bytestrømmen til TeltonikaFMCPackets forventet type.
     * TeltonikaFMCPacket bruker den rå bytestrømen som er mottatt
     *
     * @return TeltonikaFMCPacket
     */
    @Override
    protected Packet onReceive(byte[] data) {
        return new TeltonikaFmcPacket(this, data);
    }
}
