package test;

import tools.ByteStreamReader;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.BeforeEach;
import java.util.logging.Level;
import java.util.logging.Logger;
import static org.junit.jupiter.api.Assertions.*;

public class ByteStreamReaderTest {

    final Logger logger = Logger.getLogger(ByteStreamReaderTest.class.getName());

    ByteStreamReader reader;
    ByteStreamReader reader2;
    final byte[] bytes = new byte[]{0x0f, 0x00, 0x01, 0x00, 0x01, 0x01, 0x01, 0x00, 0x01, 0x00};


    @BeforeEach
    void setup() {
        reader = new ByteStreamReader(bytes);
        reader2 = new ByteStreamReader(bytes);

    }

    @Test
    void testReadInt8() {
        logger.log(Level.INFO,"Test read int8");
        assertEquals(0x0F, reader.readInt8());
        assertNotEquals(0x0E, reader2.readInt8());
        logger.log(Level.INFO,"Test OK");
    }

    @Test
    void testReadInt16() {
        logger.log(Level.INFO,"Test read int16");
        assertEquals(0x0F00, reader.readInt16());
        assertNotEquals(0x0E00, reader2.readInt16());
        logger.log(Level.INFO,"Test OK");
    }

    @Test
    void testReadInt32() {
        logger.log(Level.INFO,"Test read int32");
        assertEquals(0x0F000100, reader.readInt32());
        assertNotEquals(0x0E000100, reader2.readInt32());
        logger.log(Level.INFO,"Test OK");
    }

    @Test
    void testReadInt64() {
        logger.log(Level.INFO,"Test read int64");
        assertEquals(0x0F00010001010100L, reader.readInt64());
        assertNotEquals(0x0E00010001010100L, reader2.readInt64());
        logger.log(Level.INFO,"Test OK");
    }

    @Test
    void testRead() {
        logger.log(Level.INFO,"Test read byte array");
        assertArrayEquals(new byte[]{0x0f, 0x00, 0x01, 0x00, 0x01}, reader.readByteArray(5));
        logger.log(Level.INFO,"Test OK");
    }

    @Test
    void testSkip() {
        logger.log(Level.INFO,"Test Skip");
        reader.skip(2);
        assertEquals(0x01, reader.readInt8());
        logger.log(Level.INFO,"Test OK");
    }
}
