package test;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotEquals;


import tools.Crc;
import tools.DataConverter;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.junit.jupiter.api.Test;

public class CrcTest {

    Logger logger = Logger.getLogger(CrcTest.class.getName());

    byte[] packet = new byte[]{(byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x6F, (byte) 0x8E, (byte) 0x02, (byte) 0x00, (byte) 0x00, (byte) 0x01, (byte) 0x77, (byte) 0xB0, (byte) 0x8E, (byte) 0x93, (byte) 0xF0, (byte) 0x01, (byte) 0x0E, (byte) 0xE6, (byte) 0xB2, (byte) 0x80, (byte) 0x0E, (byte) 0xE6, (byte) 0xB2, (byte) 0x80, (byte) 0x00, (byte) 0x6E, (byte) 0x00, (byte) 0xC0, (byte) 0x05, (byte) 0x00, (byte) 0x01, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x03, (byte) 0x00, (byte) 0x01, (byte) 0x00, (byte) 0x01, (byte) 0x01, (byte) 0x00, (byte) 0x01, (byte) 0x00, (byte) 0x3C, (byte) 0x00, (byte) 0x64, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x01, (byte) 0x00, (byte) 0x47, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x98, (byte) 0x96, (byte) 0x80, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x01, (byte) 0x77, (byte) 0xB1, (byte) 0xBF, (byte) 0xC0, (byte) 0xF0, (byte) 0x01, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x03, (byte) 0x00, (byte) 0x02, (byte) 0x00, (byte) 0x01, (byte) 0x00, (byte) 0x00, (byte) 0x20, (byte) 0x0D, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x01, (byte) 0x00, (byte) 0x47, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x01, (byte) 0x86, (byte) 0x9F, (byte) 0x00, (byte) 0x00, (byte) 0x02, (byte) 0x00, (byte) 0x00, (byte) 0x3E, (byte) 0xC3};
    byte[] corruped = new byte[]{(byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x2C, (byte) 0x8E, (byte) 0x01, (byte) 0x00, (byte) 0x00, (byte) 0x01, (byte) 0x77, (byte) 0xA5, (byte) 0x07, (byte) 0xE1, (byte) 0x08, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0xB3, (byte) 0xB0, (byte) 0x00, (byte) 0x08, (byte) 0x16, (byte) 0x50, (byte) 0x00, (byte) 0x09, (byte) 0x00, (byte) 0x05, (byte) 0x04, (byte) 0x00, (byte) 0x07, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x01, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x01, (byte) 0x00, (byte) 0x09, (byte) 0xBE, (byte) 0x20, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x01, (byte) 0x00, (byte) 0x00, (byte) 0x29, (byte) 0x94};

    @Test
    void testCorrectdata() {
        assertNotEquals(Crc.crcChecker(packet) - 1, DataConverter.byteToInt32(packet, (packet.length - 4)));
        assertEquals(Crc.crcChecker(packet), DataConverter.byteToInt32(packet, (packet.length - 4)));
        assertNotEquals(Crc.crcChecker(packet) + 1, DataConverter.byteToInt32(packet, (packet.length - 4)));
        logger.log(Level.INFO, "Test Valid CRC: Worked! ");
    }

    @Test
    void testInvaliddata() {
        assertNotEquals(Crc.crcChecker(corruped), DataConverter.byteToInt32(corruped, (corruped.length - 4)));
        assertNotEquals(Crc.crcChecker(corruped), 0);
        logger.log(Level.INFO, "Test Invalid CRC: Worked! ");
    }

}
