package test;

import dataprocessing.Protocol;
import networking.teltonika.TeltonikaPacket;
import dataprocessing.teltonika.TeltonikaProtocol;

import java.net.Socket;
import java.util.logging.Level;
import java.util.logging.Logger;

import networking.teltonika.TeltonikaClient;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;


import static org.junit.jupiter.api.Assertions.*;

public class TeltonikaProtocolTest {

    Logger logger = Logger.getLogger(TeltonikaProtocolTest.class.getName());

    TeltonikaPacket packetData;
    TeltonikaPacket packetError;
    TeltonikaPacket packetError2;
    TeltonikaPacket packetHandshake;
    TeltonikaProtocol protocol = new TeltonikaProtocol();

    @BeforeEach
    void setUp() {
        byte[] packet = new byte[]{(byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x2D, (byte) 0x8E, (byte) 0x01, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x0E, (byte) 0xE6, (byte) 0xB2, (byte) 0x80, (byte) 0x11, (byte) 0xE1, (byte) 0xA3, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x01, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x01, (byte) 0x00, (byte) 0xA2, (byte) 0x00, (byte) 0x64, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x01, (byte) 0x00, (byte) 0x00, (byte) 0xF1, (byte) 0xD4};
        byte[] error = new byte[]{(byte) 0x00, (byte) 0x00, (byte) 0x00};
        byte[] error2 = new byte[]{(byte) 0x00, (byte) 0x10, (byte) 0x00, (byte) 0x00, (byte) 0x0F, (byte) 0x33};
        byte[] handshake = new byte[]{(byte) 0x00, (byte) 0x0F, (byte) 0x33, (byte) 0x35, (byte) 0x36, (byte) 0x33, (byte) 0x30, (byte) 0x37, (byte) 0x30, (byte) 0x34, (byte) 0x32, (byte) 0x34, (byte) 0x34, (byte) 0x31, (byte) 0x30, (byte) 0x31, (byte) 0x33};

        TeltonikaClient client = new TeltonikaClient(null, new Socket(), null, null);
        this.packetData = new TeltonikaPacket(client, packet);
        this.packetError = new TeltonikaPacket(client, error);
        this.packetError2 = new TeltonikaPacket(client, error2);
        this.packetHandshake = new TeltonikaPacket(client, handshake);
    }

    @Test
    void testValidateData() {
        logger.log(Level.INFO, "Test 1: Data Packet ");
        assertEquals(Protocol.Codes.DATA, protocol.validateData(packetData));
        logger.log(Level.INFO, "Test 2: Error 1 Packet ");
        assertEquals(Protocol.Codes.ERROR, protocol.validateData(packetError));
        logger.log(Level.INFO, "Test 3: Error 2 Packet ");
        assertEquals(Protocol.Codes.ERROR, protocol.validateData(packetError2));
        logger.log(Level.INFO, "Test 4: Handshake Packet ");
        assertEquals(Protocol.Codes.HANDSHAKE, protocol.validateData(packetHandshake));
    }

    @Test
    void testProcessHandshake() {
        logger.log(Level.INFO, "Test 5: Handshake Packet ");
        assertEquals(356307042441013L, protocol.processHandshake(packetHandshake));
        logger.log(Level.INFO, "Test 6: Data Packet ");
        assertEquals(-1L, protocol.processHandshake(packetData));
        logger.log(Level.INFO, "Test 7: Error 1 Packet ");
        assertEquals(-1L, protocol.processHandshake(packetError));
    }

    @Test
    void testFormatData() {
        var data = protocol.processData(packetData);
        logger.log(Level.INFO, "Test 9: Device Family ");
        assertEquals("Teltonika", data.getDeviceFamily());
        logger.log(Level.INFO, "Test 10: GPS Data ");
        assertEquals(25, data.getDataPackets().get(0).getGpsElement().getLongitude());
        assertEquals(30, data.getDataPackets().get(0).getGpsElement().getLatitude());
        logger.log(Level.INFO, "Test 10: GPS Data ");
        assertEquals("Plow", data.getDataPackets().get(0).getEquipmentArray()[0].getName());
        assertEquals("up", data.getDataPackets().get(0).getEquipmentArray()[0].getState());
    }

    @Test
    void testRawData() {
        var data = protocol.processData(packetData);
        assertEquals("Rotation of Arm", data.getDataPackets().get(0).getEquipmentArray()[0].getRawData().get(0).getName());
        assertEquals("100.0", data.getDataPackets().get(0).getEquipmentArray()[0].getRawData().get(0).getValue());
    }
}
