package tools;

import java.nio.ByteBuffer;

/**
 * Konverterer data
 */
public class DataConverter {
    
    private DataConverter(){}

    /**
     * Leser en byte fra byte-array
     *
     * @param bytes  byte-array
     * @param offset offsetten
     * @return byte
     */
    public static byte byteToInt8(byte[] bytes, int offset) {
        return bytes[offset];
    }

    /**
     * Skriver en byte til byte-array
     *
     * @param val verdien
     * @return byte-array på lendge 1
     */
    public static byte[] int8ToByte(byte val) {
        return new byte[]{val};
    }

    /**
     * Leser en short fra byte-array
     *
     * @param bytes  byte-array
     * @param offset offsetten
     * @return short
     */
    public static short byteToInt16(byte[] bytes, int offset) {
        return (short) (((bytes[offset] & 0xFF) << 8) |
                ((bytes[offset + 1] & 0xFF)));
    }

    /**
     * Skriver en short til byte-array
     *
     * @param val verdien
     * @return byte-array på lengde 2
     */
    public static byte[] int16ToByte(short val) {
        return ByteBuffer.allocate(2).putShort(val).array();
    }

    /**
     * Leser en int fra byte-array
     *
     * @param bytes  byte-array
     * @param offset offsetten
     * @return int
     */
    public static int byteToInt32(byte[] bytes, int offset) {
        return ((bytes[offset] & 0xFF) << 24) |
                ((bytes[offset + 1] & 0xFF) << 16) |
                ((bytes[offset + 2] & 0xFF) << 8) |
                ((bytes[offset + 3] & 0xFF));
    }

    /**
     * Skriver en int til byte-array
     *
     * @param val verdien
     * @return byte-array på lengde 4
     */
    public static byte[] int32ToByte(int val) {
        return ByteBuffer.allocate(4).putInt(val).array();
    }

    /**
     * Leser en long fra byte-array
     *
     * @param bytes  byte-array
     * @param offset offsetten
     * @return long
     */
    public static long byteToInt64(byte[] bytes, int offset) {
        return ((long) (bytes[offset] & 0xFF) << 56) |
                ((long) (bytes[offset + 1] & 0xFF) << 48) |
                ((long) (bytes[offset + 2] & 0xFF) << 40) |
                ((long) (bytes[offset + 3] & 0xFF) << 32) |
                ((long) (bytes[offset + 4] & 0xFF) << 24) |
                ((long) (bytes[offset + 5] & 0xFF) << 16) |
                ((long) (bytes[offset + 6] & 0xFF) << 8) |
                ((long) (bytes[offset + 7] & 0xFF));
    }

    /**
     * Skriver en long til byte-array
     *
     * @param val verdien
     * @return byte-array på lengde 8
     */
    public static byte[] int64ToByte(long val) {
        return ByteBuffer.allocate(8).putLong(val).array();
    }

}
